@@ -1,8 +1,9 @@
 /* pkginfo.c - Retrieve information about packages for deborphan.
    Copyright (C) 2000, 2001, 2002, 2003 Cris van Pelt
    Copyright (C) 2003, 2004, 2006 Peter Palfrader
+   Copyright (C) 2008 Carsten Hey
 
-   $Id: pkginfo.c 577 2006-10-05 16:33:10Z weasel $
+   $Id: pkginfo.c 710 2008-08-07 08:35:52Z carsten-guest $
 
    Distributed under the terms of the Artistic License.
 */
@@ -28,20 +29,23 @@
 
 extern int options[];
 
-static regex_t re_statusinst, re_statushold, re_namedev, re_gnugrepv,
-               re_descdummy, re_desctransit, re_statusconfig;
+static regex_t re_statusinst, re_statusnotinst, re_statushold, re_namedev,
+               re_gnugrepv, re_descdummy, re_desctransit, re_statusconfig,
+               re_status;
 
 void
 init_pkg_regex()
 {
     regcomp(&re_statusinst,
 	    "^Status:.*[^-]installed$", REG_EXTENDED | REG_FLAGS);
+    regcomp(&re_statusnotinst,
+	    "^Status:.*not\\-installed$", REG_EXTENDED | REG_FLAGS);
     regcomp(&re_statushold,
 	    "^Status:hold.*[^-]installed$", REG_EXTENDED | REG_FLAGS);
-
-    if (options[FIND_CONFIG])
-	    regcomp(&re_statusconfig,
-		"^Status:.*config\\-files$", REG_EXTENDED | REG_FLAGS);
+    regcomp(&re_statusconfig,
+	    "^Status:.*config\\-files$", REG_EXTENDED | REG_FLAGS);
+    regcomp(&re_status,
+	    "^Status:", REG_EXTENDED | REG_FLAGS);
 
     if (options[GUESS]) {
 	char guess[128];
@@ -94,8 +98,10 @@ void
 free_pkg_regex()
 {
     regfree(&re_statusinst);
+    regfree(&re_statusnotinst);
     regfree(&re_statushold);
     regfree(&re_statusconfig);
+    regfree(&re_status);
     regfree(&re_namedev);
     regfree(&re_gnugrepv);
 }
@@ -276,16 +282,24 @@ get_pkg_name(const char *line, pkg_info * package)
 void
 get_pkg_status(const char *line, pkg_info * package)
 {
-    if (!regexec(&re_statusinst, line, 0, NULL, 0))
+    if (!regexec(&re_statusinst, line, 0, NULL, 0)) {
 	set_install(package);
-
-    if (!options[FORCE_HOLD]) {
-	if (!regexec(&re_statushold, line, 0, NULL, 0))
-	    set_hold(package);
-    }
-    if (options[FIND_CONFIG]) {
-	if (!regexec(&re_statusconfig, line, 0, NULL, 0))
-	    set_config(package);
+	if (!options[FORCE_HOLD]) {
+	    if (!regexec(&re_statushold, line, 0, NULL, 0))
+		set_hold(package);
+	}
+    } else if (!regexec(&re_statusconfig, line, 0, NULL, 0)) {
+	if (options[FIND_CONFIG]) set_config(package);
+    } else if (regexec(&re_statusnotinst, line, 0, NULL, 0)) {
+	/* The package state is neither installed, config-files nor
+	 * not-installed.  It is also possible that get_pkg_info()
+	 * wrongly detected the current line as a status line.
+	 *
+	 * Abort with error message "improper state" if we
+	 * really parsed a status line.
+	 */
+	if (!regexec(&re_status, line, 0, NULL, 0))
+	    exit_improperstate();
     }
 }
 
